package genericitate;

import java.util.Collection;
import java.util.Iterator;

/**
 * 
 * @author daniel
 *
 *	Coada de prioritati. Implementata ca o lista simplu inlantuita, cu adaugare de 
 *	valori noi in pozitiile corespunzatoare.
 *
 *	Prioritatea e data de comparatia relativa a obiectelor. Astfel, daca
 *	<code><br> a.compareTo(b) < 0 <br></code> atunci a este prioritar lui b. 
 *
 * @param <T>	Tipul parametrizat, neaparat comparabil.
 */
public class PriorityQueue <T extends Comparable <T>> implements Iterable <T> {
	
	/**
	 * 
	 * @author daniel
	 *	Nod de lista simplu inlantuita
	 */
	private class Node	 {
		T value;
		Node next;
		
		Node (T val) {
			value = val;
			next = null;
		}
	}
	
	Node head = null;
	
	public PriorityQueue() {
		
	}
	
	/**
	 * Adauga un element nou in pozitia corezpunzatoare a cozii de prioritati.
	 * @param elem
	 */
	public void add (T elem) {
		// structura e goala
		if (head == null) {
			head = new Node (elem);
			return;
		}
			
		Node node = head;
		Node previous = null;
		Node newNode = new Node(elem);
		
		while (node != null && elem.compareTo(node.value) > 0) {
			previous = node;
			node = node.next;
		}
		
		// sunt la inceput, actualizez capul listei
		if (previous == null) {
			newNode.next = head;
			head = newNode;
		} else {
			// inserez nodul nou intre previous si node 
			// ! node poate fi null (=> cazul in care adaug la sfarsitul listei)
			previous.next = newNode;
			newNode.next = node;
		}
	}
	
	/**
	 * Intoarce elementul prioritar din lista, cu eliminarea lui. 
	 * @return primul element ca prioritate 
	 */
	public T pop() {
		if (head == null)
			return null;
		
		Node node = head;
		head = head.next;
		return node.value;
	}
	
	public void addAll (Collection <? extends T> collection) {
		for (T elem: collection)
			add(elem);
	}

	@Override
	public Iterator<T> iterator() {
		return new Iterator <T> () {
			Node current = head;

			@Override
			public boolean hasNext() {
				return current != null;
			}

			@Override
			public T next() {
				T val = current.value;
				current = current.next;
				return val;
			}

			@Override
			public void remove() {
				// lasat neimplementat
			}
			
		};
	}
	
	public String toString() {
		String str = "[ ";
		
		for (T elem : this) {
			str += elem.toString() + " ";
		}
		
		str += "]";
		return str;
	}
}
